/*
 * MainWindow class
 * Lots of variables, functions are included in this class
 * Copyright 2014 William Wong <librehat@outlook.com>
 */
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QCoreApplication>
#include <QDesktopWidget>
#include <QAbstractButton>
#include <QPushButton>
#include <QSystemTrayIcon>
#include <QMenu>
#include <QFileDialog>
#include <QFile>
#include <QStandardPaths>
#include <QInputDialog>
#include <QMessageBox>
#include "ssprofile.h"
#include "profiles.h"
#include "ss_process.h"
#include "ssvalidator.h"
#include "ip4validator.h"
#include "portvalidator.h"
#include "addprofiledialogue.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    Profiles *m_profile;

signals:
    void configurationChanged(bool saved = false);
    void miscConfigurationChanged(bool saved = false);

public slots:
    void startButtonPressed();

private slots:
    void onCurrentProfileChanged(int);
    void onBackendToolButtonPressed();
    void backendTypeChanged(const QString &);
    void addProfileDialogue(bool);
    void onAddProfileDialogueAccepted(const QString &, bool, const QString &);
    void onAddProfileDialogueRejected(bool enforce = false);
    void profileEditButtonClicked(QAbstractButton*);
    void stopButtonPressed();
    void deleteProfile();
    void processStarted();
    void processStopped();
    void systrayActivated(QSystemTrayIcon::ActivationReason);
    void showorhideWindow();
    void onReadReadyProcess(const QByteArray &o);
    void onConfigurationChanged(bool);
    void onMiscConfigurationChanged(bool);
    void serverEditFinished(const QString &);
    void sportEditFinished(const QString &);
    void pwdEditFinished(const QString &);
    void laddrEditFinished(const QString &);
    void lportEditFinished(const QString &);
    void methodChanged(const QString &);
    void timeoutChanged(int);
    void autoHideChecked(int);
    void autoStartChecked(int);
    void debugChecked(int);
    void miscButtonBoxClicked();
    void aboutButtonClicked();

private:
    Ui::MainWindow *ui;
    SS_Process ss_local;
    SSProfile current_profile;
    QString detectSSLocal();
    QString jsonconfigFile;
    IP4Validator ipv4addrValidator;
    PortValidator portValidator;
    QSystemTrayIcon systray;
    QMenu systrayMenu;
    AddProfileDialogue addProfileDlg;
    void saveProfile();
    void saveMiscConfig();
    void checkIfSaved();
    static const QString aboutText;

protected:
    void changeEvent(QEvent *);
    void closeEvent(QCloseEvent *);
};

#endif // MAINWINDOW_H
